function [ alpha ] = LiMapS(s, D, DINV, gamma, maxIter, ftype)
%
% [ alpha ] = LIMAPS(s, D, DINV, gamma, maxIter, ftype)
%
%   Returns the sparsest vector alpha which satisfies underdetermined 
%   system of equations  D*alpha=s, using  Lipschitzian Mapping for Sparsity
%   Recovery algorithm. 
%   The dictionary matrix  D should have more columns than rows. The 
%   number of rows of dictionary D  must be equal to the length of the 
%   column observation vector(or matrix) s.
%
%     The observations signal(s) s and the dictionary D necessarily, must 
%   be provided by the user. The other parameters have defult values, or
%   may be provided by the user.
% 
%   This algorithm work fine with matrix too, returning the sparsest matrix
%   alpha that satisfy the underdetermined system of equations D*alpha=s.
%   Then the dictionary D is of size [n x m], s must be of size [ s x T ] and
%   as consiquence the coefficients matirx alpha is of size [ m x T]
% 
%   s: 
%       The observation vector (matrix)
%        
%   D: 
%       The dictionary 
%        
%   DINV (optional): 
%       The pseudo-inverse of the dictionary matrix D
%
%   gamma (optional): 
%       The increase factor (default 1.01)
%
%   maxIter (optional): 
%       Maximum number of iterations (default 1000)
%
%   ftype (optional):
%       String that represent the shrinking function
%               - 'exp'     (default) exponential
%               - 'exp2'    exponential function
%               - 'htan'    hyperbolic function
%               - 'abs'     absolute value function
%               - 'sqrt'    square root function
%               - 'err'     error function
%               - 'gud'     Gudermannian function
%
% Authors: Alessandro Adamo and Giuliano Grossi
%
% WEB PAGE:
% ------------------
%    http://dalab.dsi.unimi.it/limaps
%
% HISTORY:
%--------------
% Version 2.0: 9 Feb. 2013
%        official version.
%

if(nargin<2||nargin>6)
    error('Wrong parameters number');
end
if(nargin==2)
    DINV=pinv(D);
    gamma=1.01;
    maxIter=1000;
    ftype='exp';
end
if(nargin==3)
    gamma=1.01;
    maxIter=1000;
    ftype='exp';
end
if(nargin==4)
    maxIter=1000;
    ftype='exp';
end
if(nargin==5)
    ftype='exp';
end


%% --  INITIALIZATION  --
alpha = DINV*s;

lambda = 1/max(abs(alpha(:)));
epsilon=1e-5;                    % stopping criteria
alpha_min=1e-2;

% choose the shrinking function
f=shrinkage_function (ftype);

%% --  CORE  --
for i=1:maxIter
    
    alphaold=alpha;
    % apply sparsity constraction mapping: increase sparsity
    beta = f(alpha,lambda);
    
    beta(abs(beta)<alpha_min)=0;
    
    % apply the orthogonal projection
    alpha = beta-DINV*(D*beta-s);
    
    % update the lambda coefficient
    lambda = lambda*gamma;
    
    % check the stopping criteria
    if (norm(alpha(:)-alphaold(:))<epsilon)
        break;
    end
   
    if (mod(i,100)) 
        if(sum(abs(f(alpha,lambda)./alpha)>1/lambda)>size(D,1))
           break;
        end
    end
    
end

%% -- REFINEMENT --

% threshold the coefficients 
alpha(abs(alpha)<alpha_min) = 0;

% final refinements of the solution
for i=1:size(alpha,2);
    alpha_ind = alpha(:,i)~=0;
    D1 = D(:,alpha_ind);
    alpha(alpha_ind,i) = alpha(alpha_ind,i) - pinv(D1)*(D1*alpha(alpha_ind,i)-s(:,i));
end

end

function [ f ] = shrinkage_function ( type )

    f = @exponential_func;
    switch(type)
        case 'exp'
            f = @exponential_func;
        case 'exp2' 
            f = @exponential2_func;
        case 'htan' 
            f = @hyperbolic_tangent_func;
        case 'abs' 
            f = @absolute_value_func;
        case 'sqrt' 
            f = @square_root_func;
        case 'err' 
            f = @error_func;
        case 'gud' 
            f = @gudermannian_func;
    end
    
end

function [ y ] = exponential_func(x,lambda)
    y=x.*(1-exp(-lambda*abs(x)));
end

function [ y ] = exponential2_func(x,lambda)
    y=x.*(2./(1+exp(-lambda*abs(x)))-1);
end

function [ y ] = hyperbolic_tangent_func(x,lambda)
    y=x.*tanh(lambda*abs(x));
end

function [ y ] = absolute_value_func(x,lambda)
    y=x.*(lambda*x.^2./(1+lambda*x.^2));
end

function [ y ] = square_root_func(x,lambda)
    y=x.*(lambda*abs(x))./sqrt(1+(lambda*x).^2);
end

function [ y ] = error_func(x,lambda)
    y=x.*erf(lambda*abs(x));
end

function [ y ] = gudermannian_func(x,lambda)
    y=x.*(2/pi*atan(sinh(pi/2*lambda*abs(x))));
end
